package com.fzm.portal.aop;

import com.alibaba.fastjson.JSON;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.lang.reflect.Parameter;
import java.time.Instant;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tangtuo
 * @date 2021/9/10 14:27
 */
@Slf4j
@Order(1)
@Component
@Aspect
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class LogAop {

    private final HttpServletRequest request;

    private static final ThreadLocal<Long> local = new ThreadLocal<>();

    @Pointcut(value = "execution(public * com.fzm.portal.controller..*.*(..))")
    public void pointCut() {
    }


    /**
     * 前置通知
     *
     * @param joinPoint
     */
    @Before("pointCut()")
    public void deBefore(JoinPoint joinPoint) {
        long start = Instant.now().toEpochMilli();
        local.set(start);
    }


    @AfterReturning(pointcut = "pointCut()")
    public void logAfterReturning(JoinPoint joinPoint) {
        log(joinPoint, null);
    }

    @AfterThrowing(pointcut = "pointCut()", throwing = "e")
    public void logAfterThrowing(JoinPoint joinPoint, Throwable e) {
        log(joinPoint, e);
    }

    private void log(JoinPoint joinPoint, Throwable e) {
        String requestURI = request.getRequestURI();

        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        Method method = signature.getMethod();
        List<Object> parameter = getParameter(method, joinPoint.getArgs());

        long duration = Instant.now().toEpochMilli() - local.get();

        String msg = duration + "ms "
                + request.getMethod() + " "
                + requestURI + " "
                + JSON.toJSONString(parameter) + (e == null ? "" : " " + e.getMessage());

        log.info(msg);
    }

    private List<Object> getParameter(Method method, Object[] args) {
        List<Object> argList = new ArrayList<>(args.length);
        Parameter[] parameters = method.getParameters();
        for (int i = 0; i < parameters.length; i++) {
            RequestBody requestBody = parameters[i].getAnnotation(RequestBody.class);
            if (requestBody != null) {
                argList.add(args[i]);
            }
            ModelAttribute modelAttribute = parameters[i].getAnnotation(ModelAttribute.class);
            if (modelAttribute != null) {
                argList.add(args[i]);
            }
            RequestParam requestParam = parameters[i].getAnnotation(RequestParam.class);
            if (requestParam != null) {
                Map<String, Object> map = new HashMap<>();
                String key = parameters[i].getName();
                if (StringUtils.isNotBlank(requestParam.value())) {
                    key = requestParam.value();
                }
                map.put(key, args[i]);
                argList.add(map);
            }
            PathVariable pathVariable = parameters[i].getAnnotation(PathVariable.class);
            if (pathVariable != null) {
                Map<String, Object> map = new HashMap<>();
                String key = parameters[i].getName();
                if (StringUtils.isNotBlank(pathVariable.value())) {
                    key = pathVariable.value();
                }
                map.put(key, args[i]);
                argList.add(map);
            }
        }
        return argList;
    }

}
